{- Copyright (c) 2015, Peter Ljung <ljung.peter@gmail.com>

Permission to use, copy, modify, and/or distribute this software for any purpose with or without fee is hereby
granted, provided that the above copyright notice and this permission notice appear in all copies.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING
ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL,
DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE
USE OR PERFORMANCE OF THIS SOFTWARE. -}

module Fifo where

import Prelude as P

-- Special fifo stack with maximum size

data Fifo a = F Int [a] deriving (Eq, Show)

mkEmpty :: Int -> Fifo a
mkEmpty size = F size []

push :: Fifo a -> a -> Fifo a
push f@(F size as) item | isFull f  = F size (item:init as)
                        | otherwise = F size (item:as)

isFull :: Fifo a -> Bool
isFull (F size as) = P.length as >= size

length :: Fifo a -> Int
length (F _ as) = P.length as

-- Note: first element is last pushed element
toList :: Fifo a -> [a]
toList (F _ as) = as
